/'--------------------------------------------------------------------
                  Nascent World Engine for FreeBASIC              
                     Copyright Vincent DeCampo 2009      

   This program is free software: you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------'/

#Define _MAKELIB_

#Include "nascentwe.bi"

'--------------- _Animation Object -----------------

Constructor _Animation (index As Integer, fStart As Integer, fStop As Integer, FPS As Integer, floop As Integer)
   this.FrameTick    = (1/FPS)
   this.StartFrame   = fStart
   this.StopFrame    = fStop      
   this.FrameLoop    = floop
   this.CurrentFrame = this.StartFrame
   this.SheetIndex   = index
End Constructor

Function _Animation.GetFrame() As Integer
   
   If pause = 0 Then
      If Timer > this.NextFrame Then
         this.NextFrame = Timer + this.FrameTick
         Return this.GetNextFrame()
      EndIf
   End If
   Return this.CurrentFrame
   
End Function
Function _Animation.GetNextFrame() As Integer
   
   this.CurrentFrame+=1
   If this.CurrentFrame > this.StopFrame Then
      If this.FrameLoop Then
         this.CurrentFrame = this.StartFrame
      Else
         this.CurrentFrame = this.StopFrame
      End If
   EndIf
   Return this.CurrentFrame
   
End Function
Function _Animation.GetPrevFrame() As Integer

   this.CurrentFrame-=1
   If this.CurrentFrame < this.StartFrame Then
      this.CurrentFrame = this.StopFrame
   EndIf
   Return this.CurrentFrame
   
End Function
Sub _Animation.Reset() 
   this.CurrentFrame = this.StartFrame
End Sub

'------------------- _Map Object -------------------

Function _Map.MakeMap (mWidth As Integer, mHeight As Integer) As Integer
	this.MapWidth  = mWidth
	this.MapHeight = mHeight
	this.MAP 		= Callocate( mWidth * mHeight * SizeOf(_MapCell) )
	this.MaxOffset = (mWidth * mHeight)
	For idx As Integer = 0 To (mWidth * mHeight)-1
	   this.MAP[idx].Frame = &hFF
	Next
   Return 1
End Function

Sub _Map.FillMap (frame As Integer, sheet As Integer, attr As Integer = -1) 
	For idx As Integer = 0 To (this.MapWidth * this.MapHeight)-1
	   If frame > -1 Then
   	   this.MAP[idx].Frame = frame
	   End If
      If sheet > -1 Then
	      this.MAP[idx].Sheet = sheet
      End If
	   If attr > -1 Then
	      this.MAP[idx].Attr = attr
	   EndIf
	Next
End Sub

Function _Map.LoadMap (filename As String, mWidth As Integer, mHeight As Integer) As Integer
	
	If this.MAP Then
      DeAlloc(this.MAP)
	EndIf
	
	If FileExists(filename) Then
	   this.MakeMap (mWidth,mHeight)
	   Var hFile = FreeFile()
	   Open filename For Binary As hFile
	      Get #hFile,,*this.MAP,this.MaxOffset
	   Close #hFile
      Return 1
	End If
	
End Function

Function _Map.SaveMap (filename As String) As Integer

   Var hFile = FreeFile()
   Open filename For Binary As hFile
      Put #hFile,,*this.MAP,this.MaxOffset
   Close #hFile
	Return 1
	
End Function

Function _Map.GetCell (TileX As Integer, TileY As Integer) As _MapCell
Dim As Integer pOffset,lw,hw 
Dim As _MapCell mapval

   mapval.Frame = &hFF

   If TileX < 0 Or TileY<0         Then Return (mapval)
   If TileX > (this.MapWidth-1)    Then Return (mapval)
   If TileY > (this.MapHeight-1)   Then Return (mapval)

   pOffset = (TileY*this.MapWidth)+TileX
   
   mapval = this.MAP[pOffset]    
      
	Return mapval

End Function

Sub _Map.SetCell (TileX As Integer, TileY As Integer, mapval As _MapCell) 
Dim As Integer pOffset, comp
 
   If TileX < 0 Or TileY < 0     Then Exit Sub
   If TileX > (this.MapWidth-1)  Then Exit Sub
   If TileY > (this.MapHeight-1) Then Exit Sub
   
   pOffset = (TileY*this.MapWidth)+TileX
   
   this.MAP[pOffset] = mapval

End Sub

Function _Map.Collision (Object As _Object) As Integer
Dim As Integer tx,ty,tv
Dim As _MapCell mapcell

   With Object

      tx = Int(.x)
      ty = Int(.y)
      mapcell = this.GetCell(tx,ty)
      
      If (mapcell.Attr And this.mapNONBLOCKING) = 0 Then
         If mapcell.Frame Then
            Return mapcell.Frame
         End If
      End If
      
      tx = Int(.x+.90)
      ty = Int(.y)
      mapcell = this.GetCell(tx,ty)
      If (mapcell.Attr And this.mapNONBLOCKING) = 0 Then
         If mapcell.Frame Then
            Return mapcell.Frame
         End If
      End If

      tx = Int(.x+.90)
      ty = Int(.y+.90)
      mapcell = this.GetCell(tx,ty)
      If (mapcell.Attr And this.mapNONBLOCKING) = 0 Then
         If mapcell.Frame Then
            Return mapcell.Frame
         End If
      End If

      tx = Int(.x)
      ty = Int(.y+.90)
      mapcell = this.GetCell(tx,ty)
      If (mapcell.Attr And this.mapNONBLOCKING) = 0 Then
         If mapcell.Frame Then
            Return mapcell.Frame
         End If
      End If
   
   End With   
   
End Function

Function _Map.BuildCell(Frame As Short, Sheet As Short, TileAttr As Integer, Animation As Integer) As _MapCell
Dim As _MapCell mapcell
   
   mapcell.Frame     = Frame
   mapcell.Sheet     = Sheet
   mapcell.Attr      = TileAttr 
   mapcell.Animation = Animation
   mapcell.Reserved2 = 0
   
   Return mapcell
   
End Function

Destructor _Map ()
   For a As Integer = 0 To MaxAnimationIdx
      If this.Animation(a) Then
         Delete this.Animation(a)
      EndIf
   Next
	DeAlloc (this.MAP)
	DeAlloc (this.MAP)
End Destructor

'------------------ _tmEngine Object ---------------

Constructor _tmEngine

   this.RenderSurface    = ImageCreate(DisplayWidth, DisplayHeight, 0)  
   this.ZoomSurface      = ImageCreate(DisplayWidth, DisplayHeight, 0)
	this.workimg1         = ImageCreate(TileWidth, TileHeight)
	this.workimg2         = ImageCreate(TileWidth, TileHeight)
	this.dspWidth         = DisplayWidth
	this.dspHeight        = DisplayHeight
	this.Camera.dspWidth  = this.dspWidth
	this.Camera.dspHeight = this.dspHeight
   this.TilesPerRow      = (this.dspWidth\TileWidth)
   this.TilesPerCol      = (this.dspHeight\TileHeight)
	
End Constructor

Constructor _tmEngine (dispWidth As Integer, dispHeight As Integer)

   this.RenderSurface    = ImageCreate(dispWidth, dispHeight, 0)   
   this.ZoomSurface      = ImageCreate(dispWidth, dispHeight, 0)
	this.workimg1         = ImageCreate(TileWidth, TileHeight)
	this.workimg2         = ImageCreate(TileWidth, TileHeight)
	this.dspWidth         = dispWidth
	this.dspHeight        = dispHeight
	this.Camera.dspWidth  = this.dspWidth
	this.Camera.dspHeight = this.dspHeight
   this.TilesPerRow      = (this.dspWidth\TileWidth)
   this.TilesPerCol      = (this.dspHeight\TileHeight)
	
End Constructor

Function _tmEngine.RenderMap () As Integer
Dim As Integer xo  = Int(this.Camera.x) 
Dim As Integer yo  = Int(this.Camera.y)
Dim As Integer xxo = TileWidth  * (this.Camera.x-xo)
Dim As Integer yyo = TileHeight * (this.Camera.y-yo)
Dim As Any Ptr dst = this.RenderSurface
Dim As Any Ptr tmp 
Dim As Integer TileX,TileY,Sheet,ScrX,ScrY
Dim As _MapCell MapCell
Dim As Integer dist = 0

#Macro TileBlock(d,x,y,clr)
   Line d,(x,y)-Step(TileWidth,TileHeight),(clr),BF
#EndMacro
#Macro TileBox(d,x,y,clr)
   Line d,(x,y)-Step(TileWidth,TileHeight),(clr),B
#EndMacro
#Macro TileCross(d,x,y,clr)
   Line d,(x,y)-Step(TileWidth,TileHeight),(clr)
   Line d,(x,y+TileHeight)-Step(TileWidth,-TileHeight),(clr)
#EndMacro

   If this.mask Then
      Line this.mask,(0,0)-Step(this.dspWidth, this.dspHeight),&hFF00FF,BF
   EndIf
   
	For y As Integer = 0 To this.dspHeight+TileHeight Step TileHeight
		For x As Integer = 0 To this.dspWidth+TileWidth Step TileWidth
			
			TileX   = xo+ScreenToTileX(x)
			TileY   = yo+ScreenToTileY(y)
         MapCell = this.Map.GetCell(TileX,TileY)
         
         If this.Map.lightsource Then
            With this.Map
               dist = Sqr( (*.lightsource.x-TileX)^2 + (*.lightsource.y-TileY)^2 )               
            End With
         EndIf

         ScrX = x-xxo
         ScrY = y-yyo

         TileBlock (dst,ScrX,ScrY,&h909090)
         TileBox   (dst,ScrX,ScrY,&hFF0000)
         
         Do

         /'
            Boundary Mask
         '/                  
         If this.mask Then
            If MapCell.Frame = &hFF Then
               TileBlock(this.mask,ScrX,ScrY,0)
            EndIf            
         EndIf

         /'
            Animated Tile
         '/         
         If MapCell.Attr And this.Map.mapANIMATE Then
            
            Select Case dist
               Case 0 To MaxLightDistance
                  If this.Map.Animation(mapcell.Animation) Then
                     Sheet   = this.Map.Animation(mapcell.Animation)->SheetIndex
                     this.Sprites.SheetPut(dst, Sheet, x-xxo, y-yyo, _
                              this.Map.Animation(mapcell.Animation)->GetFrame,1)
                  End If
               Case Else
                  TileBlock(dst,ScrX,ScrY,0)
            End Select
            
         Else

            /'
               Single Light Source
            '/         
            If dist > MaxLightDistance Then
               TileBlock(dst,ScrX,ScrY,0)
               Exit Do
            EndIf

            /'
               Static Tile 
            '/                        
            Select Case MapCell.Frame
               Case &hFF   '<- Out of Bounds
                  If this.map.BoundaryStyle < 0 Then
                     If this.map.BoundaryGrid Then
                        TileBlock (dst,ScrX,ScrY,0)
                        TileBox   (dst,ScrX,ScrY,&hFF0000)
                     End If
                  Else
                     tmp = this.Sprites.GetImage(this.map.BoundaryStyle)
                     If tmp Then
                        Put dst,(ScrX,ScrY),tmp,TRANS
                     Else
                        TileBlock (dst,ScrX,ScrY,0)
                        TileBox   (dst,ScrX,ScrY,&hFF0000)
                     End If
                  End If             
               Case Else   '<- Static Map Tile Image
                  If (MapCell.Attr And this.Map.mapINVISIBLE) = 0 Then
                     sheet = MapCell.Sheet
                     this.Sprites.SheetPut(dst, Sheet, ScrX, ScrY, _
                                             MapCell.Frame,1)
                  End If
                  
            End Select
            
         End If
         
         If this.DesignMode Then
            If (MapCell.Attr And this.Map.mapNonBlocking) = 0 Then
               TileCross (dst,ScrX,ScrY,&hFFFF00)
            End If
         EndIf
         
         Loop Until 1
            
		Next
	Next
	
	Return 1
	
End Function

Function _tmEngine.RenderObjects () As Integer
Dim As Integer xo  = Int(this.Camera.x) 
Dim As Integer yo  = Int(this.Camera.y)
Dim As Integer xxo = TileWidth  * (this.Camera.x-xo)
Dim As Integer yyo = TileHeight * (this.Camera.y-yo)
Dim As Integer TileImg,sx,sy,sxo,syo,drx,dry,sheet
Dim As Any Ptr dst = this.RenderSurface

	For o As Integer = 0 To MaxObjectIdx
      If this.Object(o).active Then
         If this.IsOnScreen ( this.Object(o).x, this.Object(o).y ) Then
            
            TileImg = this.Object(o).imgIdx
            
            sx  = Int(this.object(o).x)
            sy  = Int(this.object(o).y)
            sxo = TileWidth  * (this.object(o).x-sx)
            syo = TileHeight * (this.object(o).y-sy)

            drx = (TileXToScreen(sx)+sxo)-TileXToScreen(xo)-xxo
            dry = (TileYToScreen(sy)+syo)-TileYToScreen(yo)-yyo
            
            Line this.workimg1,(0,0)-Step(TileWidth,TileHeight),&hFF00FF,BF
            
            If this.Object(o).Animation Then
               
               sheet = this.Object(o).Animation->SheetIndex
               
               If this.Object(o).rotangle Then
                  
                  this.Sprites.SheetPut(this.workimg2, sheet,0,0, _
                                        this.Object(o).Animation->GetFrame)

                  RotoZoom ( this.workimg1, this.workimg2, 0, 0, this.Object(o).rotangle ,1 )
                  
                  Put dst,(drx,dry),this.workimg1,TRANS

               Else

                  this.Sprites.SheetPut(dst, sheet, drx, dry, _
                                        this.Object(o).Animation->GetFrame)

               EndIf
               
            Else
            
               Select Case TileImg
                  
                  Case -1
                     
                     Line dst,(drx,dry)-Step(TileWidth,TileHeight),&hFFFFFF,B
                                    
                  Case Else
                     
                     If this.Object(o).rotangle Then
                        
                        RotoZoom ( this.workimg1, this.Sprites.GetImage(TileImg), TileWidth_2, TileHeight_2, this.Object(o).rotangle,1,1 )
                                                 
                        Put dst,(drx,dry),this.workimg1,TRANS
                        
                     Else
                        
                        Put dst,(drx,dry),this.Sprites.GetImage(TileImg),TRANS
                        
                     End If
                     
               End Select
            
            End If
            
         End If
      EndIf
	Next
	Return 1
	
End Function

Function _tmEngine.IsOnScreen (x As Single, y As Single) As Integer

	If x < this.Camera.x-1  Then Return (0)			   
	If y < this.Camera.y-1  Then Return (0)
	If x > this.Camera.x+TilesPerRow Then Return (0)
	If y > this.Camera.y+TilesPerCol Then Return (0)
	
	Return 1
   
End Function

Function _tmEngine.GetMouseTile(ByRef x As Integer, ByRef y As Integer) As Integer
Dim As Integer mx,my,wh,bt,clip
   
   GetMouse mx,my,wh,bt,clip
   If Not clip Then
      x = ScreenToTileX(mx-This.RenderOffsetX) + (this.Camera.x) '- ScreenToTileX(This.RenderOffsetX)
      y = ScreenToTileY(my-This.RenderOffsetY) + (this.Camera.y) '- ScreenToTileY(This.RenderOffsetY)
   End If
   
   Return bNot(clip)
   
End Function

Function _tmEngine.HighlightTile (x As Integer, y As Integer) As Integer
Dim As Integer xo  = Int(this.Camera.x) 
Dim As Integer yo  = Int(this.Camera.y)
Dim As Integer xxo = TileWidth  * (this.Camera.x-xo)
Dim As Integer yyo = TileHeight * (this.Camera.y-yo)
Dim As Any Ptr dst = this.RenderSurface
Dim As Integer drx,dry

   drx = TileXToScreen(x)-TileXToScreen(xo)-xxo
   dry = TileYToScreen(y)-TileYToScreen(yo)-yyo
   Line dst,(drx,dry)-Step(TileWidth,TileHeight),Rnd*&hFFFFFF,B               
	'Draw String dst,(this.dspWidth/2,5), x & "," & y
	
	Return 1
	
End Function

Sub _tmEngine.ShowMouse () 
Dim As Integer x,y
   If this.GetMouseTile(x,y) Then
      this.HighlightTile(x,y)
   EndIf
End Sub

Function _tmEngine.RenderFog () As Integer
	If this.Fog Then
	   Put this.RenderSurface,(0,0),this.foglayer,ALPHA,this.Fog
	EndIf   
	Return this.Fog
End Function

Function _tmEngine.RenderSceneAt(x As Integer, y As Integer, zoom As Single=1.0f) As Integer
   
   Select Case zoom
      
      Case Is <= 1.0
         Put (x,y),this.RenderSurface,PSet
         this.RenderOffsetX = x
         this.RenderOffsetY = y
         Return 1
      Case Else
         RotoZoom(this.ZoomSurface,this.RenderSurface,this.dspWidth/2,this.dspHeight/2,0,zoom)
         Put (x,y),this.ZoomSurface,PSet
         this.RenderOffsetX = x
         this.RenderOffsetY = y
         Return 1
   
   End Select
   
End Function

Function _tmEngine.RenderLayer(layer As _CustomLayer) As Integer
Dim As FB.IMAGE Ptr img = layer.image
Dim As Integer aPos = Abs(layer.position)
   
   If img Then
      
      If layer.alpha Then
         Put this.RenderSurface,(0,0),img,(aPos,0)-(img->Width,img->height),ALPHA,layer.alpha
         Put this.RenderSurface,(img->Width-aPos,0),img,(0,0)-Step(img->Width,img->height),ALPHA,layer.alpha
      Else
         Put this.RenderSurface,(0,0),img,(aPos,0)-(img->Width,img->height),TRANS
         Put this.RenderSurface,(img->Width-aPos,0),img,(0,0)-Step(img->Width,img->height),TRANS
      EndIf
      
      layer.position -= layer.speed
      If Abs(layer.position) >= img->Width Then
         layer.position = 0
      EndIf
   
   Else
   
      Line this.RenderSurface,(0,0)-Step(this.dspWidth,this.dspHeight),layer.color,BF
      
   End If
   
   Return 1
   
End Function

Function _tmEngine.RenderMask() As Integer
   If this.mask Then
      Put this.RenderSurface,(0,0),this.mask,TRANS
      Return 1
   End If
End Function

Property _tmEngine.SetFog (fogval As Integer)
   
   this.Fog = IIf(fogval>255,255,fogval)
   this.Fog = IIf(fogval<0,0,this.Fog)
   
   If this.foglayer Then
      ImageDestroy this.foglayer
   EndIf
   If this.Fog Then
      this.foglayer = ImageCreate(this.dspWidth,this.dspHeight,&h90c090)
   End If
   
End Property

Property _tmEngine.EnableMask (enable As Integer)

   If this.mask Then
      ImageDestroy this.mask
   EndIf
   
   If enable Then
      this.mask = ImageCreate(this.dspWidth,this.dspHeight,0)
   EndIf

End Property

Destructor _tmEngine()
   ImageDestroy (this.RenderSurface)   
   ImageDestroy (this.ZoomSurface)
	ImageDestroy (this.workimg1)
	ImageDestroy (this.workimg2)
   If this.mask Then
      ImageDestroy this.mask
   EndIf	       
End Destructor
'------------------ _Camera Object ----------------

Sub _Camera.MoveLeft(Speed As Single = .02)
	this.x-=Speed
End Sub

Sub _Camera.MoveRight(Speed As Single = .02)
	this.x+=Speed
End Sub

Sub _Camera.MoveUp(Speed As Single = .02)
	this.y-=Speed
End Sub

Sub _Camera.MoveDown(Speed As Single = .02)
	this.y+=Speed
End Sub

Sub _Camera.CameraSet(x As Single, y As Single)
   this.x = x
   this.y = y
End Sub

Sub _Camera.CenterOn(Object As _Object)
   this.x = Object.x - (this.dspWidth\TileWidth)\2
   this.y = Object.y - (this.dspHeight\TileHeight)\2
End Sub

'------------------ _Object Object ----------------
Sub _Object.MoveLeft(Speed As Single = DefaultSpeed)
   this.lastx = x
	this.x-=Speed
End Sub

Sub _Object.MoveRight(Speed As Single = DefaultSpeed)
   this.lastx = x
	this.x+=Speed
End Sub

Sub _Object.MoveUp(Speed As Single = DefaultSpeed)
   this.lasty = y
	this.y-=Speed	
End Sub

Sub _Object.MoveDown(Speed As Single = DefaultSpeed)
   this.lasty = y
	this.y+=Speed
End Sub

Sub _Object.SetObject(x As Single, y As Single)
   this.lastx = x
   this.lasty = y
   this.x = x
   this.y = y
End Sub

Function _Object.DistanceTo( Object As _Object) As Double
   Return Sqr ( (this.x-Object.x)^2+(This.y-Object.y)^2 )   
End Function

Function _Object.CollidedWith(Object As _Object) As Integer
   
   If Abs(this.x-Object.x) > TileWidth  Then Return 0
   If Abs(this.y-Object.y) > TileHeight Then Return 0   

   Return 1
      
End Function

Destructor _Object
   If this.Animation Then
      Delete this.Animation
   EndIf
End Destructor

'------------------' _Sprites Object ---------------

Constructor _Sprites()
	this.imgStore = Callocate(MaxSpriteIdx*4)
End Constructor

Function _Sprites.LoadSprite (index As Integer, filename As String) As Any Ptr
Dim As Any Ptr newImg

	newImg = LoadBitmap (filename)
	
	If newImg Then
		this.imgCount+=1
		this.imgStore[index] = Cast(Integer,newImg)
	EndIf

	Return newImg	'Return pointer to image
	
End Function

Function _Sprites.GetImage (index As Integer) As Any Ptr
	
	If index < 0 Or index > MaxSpriteIdx Then Exit Function
	Return Cast( Any Ptr, this.imgStore[index] )
	
End Function

Function _Sprites.LoadSheet (index As Integer, filename As String, TileDx As Integer, TileDy As Integer) As Integer
Dim As Any Ptr newImg
Dim As FB.IMAGE Ptr pImg

	newImg = LoadBitmap (filename)
	
	If newImg Then
      With this.Sheet(index)
         If .SpriteSheet Then
            ImageDestroy .SpriteSheet
         EndIf 
         .SpriteSheet = newimg
         pImg = newimg
         .Cols = Int(pImg->Width  \ TileDx)
         .Rows = Int(pImg->Height \ TileDy)   
         .SpriteWidth  = TileDx
         .SpriteHeight = TileDy  
         .SheetWidth   = pImg->Width
         .SheetHeight  = pImg->Height
	      Return 1
      End With
	End If
	
	Return 0
	
End Function

Sub _Sprites.SheetPut (dst As Any Ptr, index As Integer, x As Integer, y As Integer, fnum As Integer, method As Integer = 1) 
Dim As Integer sx,sy

   If this.Sheet(index).SpriteSheet Then
      With this.Sheet(index)
         sx  = Int(fnum Mod .Cols) * .SpriteWidth 
         sy  = Int(fnum \ .Cols) * .SpriteHeight
         
         Select Case Method
            Case 0 'PSET
               Put dst, (x,y),.SpriteSheet,(sx,sy)-Step(.SpriteWidth-1,.SpriteHeight-1),PSet
            Case 1 'TRANS
               Put dst, (x,y),.SpriteSheet,(sx,sy)-Step(.SpriteWidth-1,.SpriteHeight-1),TRANS
         End Select
      End With
   End If
   
End Sub

Destructor _Sprites()

	For i As Integer = 0 To this.imgCount-1
		If this.imgStore[i] Then
			ImageDestroy Cast( Any Ptr, this.imgStore[i] )
		EndIf
	Next
	DeAlloc (this.imgStore)

   For i As Integer = 0 To MaxSheetIdx
      If this.Sheet(i).SpriteSheet Then
         ImageDestroy this.Sheet(i).SpriteSheet
      EndIf
   Next i
   
End Destructor

'------------ _GameTick --------------

Constructor _GameClock (FPS As Integer)
   this.FrameTick = (1/FPS)
End Constructor

Function _GameClock.TickTime() As Integer
Dim As Double StopTime = Timer
   
   If StopTime-FPSCounter > 1 Then
      this.FPS        = fcount
      this.fcount     = 0
      this.FPSCounter = StopTime
   End If

   If StopTime > this.NextFrame Then
	   this.fcount+=1
      this.NextFrame = StopTime+this.FrameTick
      Return 1
   EndIf
      
   Return 0
   
End Function

'------------- _Custom Layer ----------------
'
Property _CustomLayer.LoadLayer (filename As String)

   If this.image Then
      ImageDestroy this.image
   EndIf
   
   Select Case UCase(Right(filename,4))
      Case ".BMP"
         this.image = LoadBitmap(filename)   
      Case ".PNG"
         this.image = png_load(filename,PNG_TARGET_FBNEW)
   End Select
   
   
End Property

